@@ -1,7 +1,7 @@
 import asyncio
 import builtins
 import os
-from typing import Any, Set, Text, Optional, MutableMapping
+from typing import Any, Set, Text, Optional, MutableMapping, MutableSequence
 
 import dill
 from IPython.core.compilerop import CachingCompiler
@@ -16,12 +16,6 @@ from jupyter_workflow.streamflow.command import JupyterCommandOutput
 
 class ControlTokenProcessor(DefaultTokenProcessor):
 
-    def __init__(self,
-                 port: Port,
-                 name: Text):
-        super().__init__(port)
-        self.name: Text = name
-
     async def compute_token(self, job: Job, command_output: JupyterCommandOutput) -> Token:
         return Token(name=self.port.name, value=None, job=job.name)
 
@@ -30,11 +24,9 @@ class FileTokenProcessor(DefaultTokenProcessor):
 
     def __init__(self,
                  port: Port,
-                 name: Optional[Text] = None,
                  value: Optional[Text] = None,
                  value_from: Optional[Text] = None):
         super().__init__(port)
-        self.name: Optional[Text] = name
         self.value: Optional[Text] = value
         self.value_from: Optional[Text] = value_from
 
@@ -42,7 +34,7 @@ class FileTokenProcessor(DefaultTokenProcessor):
         context = self.port.step.context
         path_processor = get_path_processor(self.port.step)
         src_job = context.scheduler.get_job(token.job)
-        src_path = token.value['src']
+        src_path = token.value
         if not path_processor.isabs(src_path):
             src_path = path_processor.join(src_job.output_directory, src_path)
         dest_path = os.path.join(output_dir, path_processor.basename(src_path))
@@ -54,15 +46,11 @@ class FileTokenProcessor(DefaultTokenProcessor):
             dst_job=None,
             writable=True)
         # Update token
-        return token.update({**token.value, **{'dst': dest_path}})
+        return token.update(dest_path)
 
     async def compute_token(self, job: Job, command_output: JupyterCommandOutput) -> Token:
-        token_value = {
-            'name': self.name,
-            'type': 'file',
-            'src': (self.value if self.value is not None else
-                    command_output.user_ns.get(self.value_from))
-        }
+        token_value = (self.value if self.value is not None else
+                       command_output.user_ns.get(self.value_from))
         return Token(name=self.port.name, value=token_value, job=job.name)
 
     async def _register_data(self, job: Job, path: Text):
@@ -83,19 +71,18 @@ class FileTokenProcessor(DefaultTokenProcessor):
         data_locations = set()
         for resource in resources:
             data_locations.update(context.data_manager.get_data_locations(resource, token.value))
-        resources.update({l.resource for l in data_locations})
+        resources.update({loc.resource for loc in data_locations})
         return resources
 
     async def update_token(self, job: Job, token: Token) -> Token:
-        filepath = token.value['src']
         path_processor = get_path_processor(self.port.step)
-        dest_path = path_processor.join(job.input_directory, os.path.basename(filepath))
+        dest_path = path_processor.join(job.input_directory, os.path.basename(token.value))
         await self.port.step.context.data_manager.transfer_data(
-            src=filepath,
+            src=token.value,
             src_job=None,
             dst=dest_path,
             dst_job=job)
-        return token.update({**token.value, **{'dst': dill.dumps(dest_path)}})
+        return token.update(dill.dumps(dest_path))
 
     async def weight_token(self, job: Job, token_value: Any) -> int:
         if job is not None and job.get_resources():
@@ -111,43 +98,34 @@ class NameTokenProcessor(DefaultTokenProcessor):
 
     def __init__(self,
                  port: Port,
-                 name: Text,
-                 token_type: Text,
                  compiler: CachingCompiler,
                  serializer: MutableMapping[Text, Any] = None,
                  value: Optional[Text] = None,
                  value_from: Optional[Text] = None):
         super().__init__(port)
-        self.name: Text = name
-        self.token_type: Text = token_type
         self.compiler: CachingCompiler = compiler
         self.serializer: MutableMapping[Text, Any] = serializer
         self.value: Optional[Text] = value
         self.value_from: Optional[Text] = value_from
 
     async def collect_output(self, token: Token, output_dir: Text) -> Token:
-        return token.update({
-            'name': self.name,
-            'type': self.token_type,
-            'serializer': self.serializer,
-            'value': executor.postload(
-                compiler=self.compiler,
-                name=self.name,
-                value=dill.loads(token.value['value']),
-                serializer=self.serializer)})
+        if isinstance(token.job, MutableSequence):
+            return await super().collect_output(token, output_dir)
+        return token.update(executor.postload(
+            compiler=self.compiler,
+            name=self.port.name,
+            value=([dill.loads(v) for v in token.value]
+                   if isinstance(token.value, MutableSequence) else dill.loads(token.value)),
+            serializer=self.serializer))
 
     async def compute_token(self, job: Job, command_output: JupyterCommandOutput) -> Token:
-        token_value = {
-            'name': self.name,
-            'type': self.token_type,
-            'value': dill.dumps(executor.predump(
-                compiler=self.compiler,
-                name=self.name,
-                value=(self.value if self.value is not None else
-                       command_output.user_ns[self.value_from] if self.value_from in command_output.user_ns else
-                       builtins.__dict__.get(self.value_from)),
-                serializer=self.serializer), recurse=True)
-        }
-        if self.serializer is not None:
-            token_value['serializer'] = self.serializer
+        value = executor.predump(
+            compiler=self.compiler,
+            name=self.port.name,
+            value=(self.value if self.value is not None else
+                   command_output.user_ns[self.value_from] if self.value_from in command_output.user_ns else
+                   builtins.__dict__.get(self.value_from)),
+            serializer=self.serializer)
+        token_value = ([dill.dumps(v, recurse=True) for v in value] if isinstance(value, MutableSequence) else
+                       dill.dumps(value, recurse=True))
         return Token(name=self.port.name, value=token_value, job=job.name)
